import json
import os

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


class TestConflictDiamondTest:
    def test_version_diamond_conflict(self):
        """
        test that we obtain a version conflict with a diamond, and that we can fix it by
        defining an override in the "game" consumer
        game -> engine/1.0 -> math/1.0
          |---> ai/1.0 -----> math/1.0.1 (conflict)
        """
        c = TestClient(light=True)
        c.save({"math/conanfile.py": GenConanfile("math"),
                "engine/conanfile.py": GenConanfile("engine", "1.0").with_requires("math/1.0"),
                "ai/conanfile.py": GenConanfile("ai", "1.0").with_requires("math/1.0.1"),
                "game/conanfile.py": GenConanfile("game", "1.0").with_requires("engine/1.0",
                                                                               "ai/1.0"),
                })
        c.run("create math --version=1.0")
        c.run("create math --version=1.0.1")
        c.run("create math --version=1.0.2")
        c.run("create engine")
        c.run("create ai")
        c.run("install game", assert_error=True)
        assert "Version conflict: Conflict between math/1.0.1 and math/1.0 in the graph" in c.out
        # This shouldnt error, so we are able to diagnose our dependency graph
        # The UX still need to be improved, but this is start
        c.run("graph info game --filter=requires", assert_error=True)
        assert "math/1.0" in c.out

        def _game_conanfile(version, reverse=False):
            if reverse:
                """
                 game ---(override)--_> math/newversion
                    |---> engine/1.0 -> math/1.0
                    |---> ai/1.0 -----> math/1.0.1 (conflict solved by override)
                """
                return GenConanfile("game", "1.0")\
                    .with_requirement(f"math/{version}", override=True)\
                    .with_requirement("engine/1.0")\
                    .with_requirement("ai/1.0")
            else:
                """
                game --> engine/1.0 -> math/1.0
                   |---> ai/1.0 -----> math/1.0.1 (conflict solved by override)
                   |---(override)--_> math/newversion
                """
                return GenConanfile("game", "1.0").with_requirement("engine/1.0") \
                    .with_requirement("ai/1.0") \
                    .with_requirement(f"math/{version}", override=True)

        for v in ("1.0", "1.0.1", "1.0.2"):
            c.save({"game/conanfile.py": _game_conanfile(v)})
            c.run("install game")
            c.assert_overrides({"math/1.0": [f"math/{v}"],
                                "math/1.0.1": [f"math/{v}"]})
            c.assert_listed_require({f"math/{v}": "Cache"})

        # Check that order of requirements doesn't affect
        for v in ("1.0", "1.0.1", "1.0.2"):
            c.save({"game/conanfile.py": _game_conanfile(v, reverse=True)})
            c.run("install game")
            c.assert_overrides({"math/1.0": [f"math/{v}"],
                                "math/1.0.1": [f"math/{v}"]})
            c.assert_listed_require({f"math/{v}": "Cache"})

        c.run("install --requires=engine/1.0  --requires=ai/1.0", assert_error=True)
        assert "Conflict between math/1.0.1 and math/1.0 in the graph" in c.out
        assert "Conflict originates from ai/1.0" in c.out


@pytest.mark.parametrize("version_range", [True, False])
def test_conflict_user(version_range):
    # https://github.com/conan-io/conan/issues/17875
    v = "[^1.0]" if version_range else "1.0"
    c = TestClient(light=True)
    c.save({"dep/conanfile.py": GenConanfile("dep", "1.0"),
            "pkg/conanfile.py": GenConanfile("pkg", "1.0").with_requires(f"dep/{v}@user1"),
            "app/conanfile.py": GenConanfile("app", "1.0").with_requires(f"pkg/{v}@user1",
                                                                         f"dep/{v}@user2")})
    c.run("create dep --user=user1")
    c.run("create dep --user=user2")
    c.run("create pkg --user=user1")
    c.run("install app", assert_error=True)
    assert f"Version conflict: Conflict between dep/{v}@user1 and dep/{v}@user2" in c.out


def test_conflict_user_order():
    # https://github.com/conan-io/conan/issues/17875
    c = TestClient(light=True)
    c.save({"dep/conanfile.py": GenConanfile("dep", "1.0"),
            "pkg/conanfile.py": GenConanfile("pkg", "1.0").with_requires("dep/1.0@user1"),
            "app/conanfile.py": GenConanfile("app", "1.0").with_requires("pkg/1.0@user1",
                                                                         "dep/[>=1.0]@user2")})
    c.run("create dep --user=user1")
    c.run("create dep --user=user2")
    c.run("create pkg --user=user1")
    c.run("install app", assert_error=True)
    assert "ERROR: Version conflict: Conflict between dep/1.0@user1 and dep/[>=1.0]@user2" in c.out


@pytest.mark.parametrize("test", [True, False])
@pytest.mark.parametrize("order", [True, False])
class TestErrorVisibleFalse:

    def test_subgraph_conflict(self, order, test):
        #  cli--> pkg1/1.0 -(visible=False) --------------> pkg3/1.0 (conflict)
        #             \----> pkg2/1.0 --------------------> pkg3/1.1 (conflict)
        # This conflict is good, the default dependencies are incompatible in definition
        tc = TestClient(light=True)
        pkg1 = GenConanfile("pkg1", "1.0")
        if order:
            pkg1.with_requirement("pkg3/1.0", visible=False, test=test).with_requirement("pkg2/1.0")
        else:
            pkg1.with_requirement("pkg2/1.0").with_requirement("pkg3/1.0", visible=False, test=test)
        tc.save({"pkg3/conanfile.py": GenConanfile("pkg3"),
                 "pkg2/conanfile.py": GenConanfile("pkg2", "1.0").with_requirement("pkg3/1.1"),
                 "pkg1/conanfile.py": pkg1})
        tc.run("export pkg3 --version=1.0")
        tc.run("export pkg3 --version=1.1")
        tc.run("export pkg2")
        # Creating this pkg1 does generate a conflict
        tc.run("export pkg1")
        tc.run("graph info --requires=pkg1/1.0", assert_error=True)
        assert "ERROR: Version conflict: Conflict between pkg3/1.1 and pkg3/1.0" in tc.out

    def test_subgraph_conflict_second_level(self, order, test):
        #  cli--> pkg1/1.0 -(visible=False) ---> gtest/1.0 -> zlib/1.0
        #             \----> boost/1.0 ---------------------> zlib/1.1 (conflict)
        # This conflict is good, the default dependencies are incompatible in definition
        tc = TestClient(light=True)
        pkg = GenConanfile("pkg1", "1.0")
        if order:
            pkg.with_requirement("gtest/1.0", visible=False, test=test).with_requirement("boost/1.0")
        else:
            pkg.with_requirement("boost/1.0").with_requirement("gtest/1.0", visible=False, test=test)
        tc.save({"zlib/conanfile.py": GenConanfile("zlib"),
                 "boost/conanfile.py": GenConanfile("boost", "1.0").with_requirement("zlib/1.1"),
                 "gtest/conanfile.py": GenConanfile("gtest", "1.0").with_requirement("zlib/1.0"),
                 "pkg1/conanfile.py": pkg})
        tc.run("export zlib --version=1.0")
        tc.run("export zlib --version=1.1")
        tc.run("export boost")
        tc.run("export gtest")
        # Creating this pkg1 does generate a conflict
        tc.run("export pkg1")
        tc.run("graph info --requires=pkg1/1.0", assert_error=True)
        if order:
            assert "ERROR: Version conflict: Conflict between zlib/1.1 and zlib/1.0" in tc.out
            assert "Conflict originates from boost/1.0" in tc.out
        else:
            assert "ERROR: Version conflict: Conflict between zlib/1.0 and zlib/1.1" in tc.out
            assert "Conflict originates from gtest/1.0" in tc.out

    def test_subgraph_no_conflict(self, order, test):
        #  cli--> pkg1/1.0 -(visible=False) --------------> pkg3/1.0 (no conflict)
        #             \----> pkg2/1.0 --------------------> pkg3/1.0 (no conflict)
        # This doesn't conflict, but package topology is affected, converging to a direct dependency
        # of a visible one
        #  cli--> pkg1/1.0 -(visible=True) --------------> pkg3/1.0 (no conflict)
        #             \----> pkg2/1.0 ----------------------/
        tc = TestClient(light=True)
        pkg1 = GenConanfile("pkg1", "1.0")
        if order:
            pkg1.with_requirement("pkg3/1.0", visible=False, test=test).with_requirement("pkg2/1.0")
        else:
            pkg1.with_requirement("pkg2/1.0").with_requirement("pkg3/1.0", visible=False, test=test)
        tc.save({"pkg3/conanfile.py": GenConanfile("pkg3"),
                 "pkg2/conanfile.py": GenConanfile("pkg2", "1.0").with_requirement("pkg3/1.0"),
                 "pkg1/conanfile.py": pkg1})
        tc.run("export pkg3 --version=1.0")
        tc.run("export pkg2")
        tc.run("export pkg1")

        tc.run("graph info --requires=pkg1/1.0 --format=json")
        if not test:
            assert "WARN: risk: Packages required both with visible=True and visible=False" in tc.out
            assert "pkg3/1.0: Required by pkg1/1.0" in tc.out
        else:
            assert "WARN: risk" not in tc.out
        graph = json.loads(tc.stdout)
        assert len(graph["graph"]["nodes"]) == 4  # Including the CLI 0-3
        pkg1 = graph["graph"]["nodes"]["1"]
        deps = pkg1["dependencies"]
        assert len(deps) == 2
        assert "pkg1/1.0" in pkg1["ref"]
        if order:
            dep_pkg2 = deps["3"]
            dep_pkg3 = deps["2"]
        else:
            dep_pkg2 = deps["2"]
            dep_pkg3 = deps["3"]
        assert dep_pkg2["ref"] == "pkg2/1.0"
        assert dep_pkg2["visible"] is True
        assert dep_pkg3["ref"] == "pkg3/1.0"
        assert dep_pkg2["visible"] is True

    def test_subgraph_no_conflict_second_level(self, order, test):
        #  cli--> pkg1/1.0 -(visible=False) ---> gtest/1.0 -> zlib/1.0
        #             \----> boost/1.0 ---------------------> zlib/1.0 (noconflict)
        tc = TestClient(light=True)
        pkg = GenConanfile("pkg1", "1.0")
        if order:
            pkg.with_requirement("gtest/1.0", visible=False, test=test).with_requirement("boost/1.0")
        else:
            pkg.with_requirement("boost/1.0").with_requirement("gtest/1.0", visible=False, test=test)
        tc.save({"zlib/conanfile.py": GenConanfile("zlib"),
                 "boost/conanfile.py": GenConanfile("boost", "1.0").with_requirement("zlib/1.0"),
                 "gtest/conanfile.py": GenConanfile("gtest", "1.0").with_requirement("zlib/1.0"),
                 "pkg1/conanfile.py": pkg})
        tc.run("export zlib --version=1.0")
        tc.run("export boost")
        tc.run("export gtest")
        # Creating this pkg1 does generate a conflict
        tc.run("export pkg1")
        tc.run("graph info --requires=pkg1/1.0 --format=json")
        assert "WARN: risk: Packages required both with visible=True and visible=False" not in tc.out
        graph = json.loads(tc.stdout)
        assert len(graph["graph"]["nodes"]) == 5  # Including the CLI 0-4
        pkg1 = graph["graph"]["nodes"]["1"]
        deps = pkg1["dependencies"]
        assert len(deps) == 3
        zlib = deps["3"]
        assert "zlib/1.0" in zlib["ref"]
        assert zlib["visible"] is True

    def test_transitive_conflict(self, order, test):
        # cli --------------------------------------------> pkg3/1.1
        #   \--> pkg1/1.0 -(visible=False) -> pkg3/1.0 (conflict)
        #             \----> pkg2/1.0 --------------------> pkg3/1.1 (no conflict)
        tc = TestClient(light=True)
        pkg1 = GenConanfile("pkg1", "1.0")
        if order:
            pkg1.with_requirement("pkg3/1.0", visible=False, test=test).with_requirement("pkg2/1.0")
        else:
            pkg1.with_requirement("pkg2/1.0").with_requirement("pkg3/1.0", visible=False, test=test)
        tc.save({"pkg3/conanfile.py": GenConanfile("pkg3"),
                 "pkg2/conanfile.py": GenConanfile("pkg2", "1.0").with_requirement("pkg3/1.1"),
                 "pkg1/conanfile.py": pkg1})
        tc.run("export pkg3 --version=1.0")
        tc.run("export pkg3 --version=1.1")
        tc.run("export pkg2")
        tc.run("export pkg1")
        tc.run("graph info --requires=pkg3/1.1 --requires=pkg1/1.0 --format=html", assert_error=True,
               redirect_stdout="graph.html")
        # Check that the graph.html is generated
        assert os.path.exists(os.path.join(tc.current_folder, "graph.html"))
        if order:
            assert "ERROR: Version conflict: Conflict between pkg3/1.1 and pkg3/1.0" in tc.out
        else:
            assert "ERROR: Version conflict: Conflict between pkg3/1.0 and pkg3/1.1" in tc.out
        assert "Conflict originates from pkg1/1.0" in tc.out

    def test_transitive_conflict_second_level(self, order, test):
        # cli --------------------------------------------> zlib/1.1
        #   \--> pkg1/1.0 -(visible=False) -gtest---------> zlib/1.0 (conflict)
        #             \----> boost/1.0 -------------------> zlib/1.1 (no conflict)
        tc = TestClient(light=True)
        pkg = GenConanfile("pkg1", "1.0")
        if order:
            pkg.with_requirement("gtest/1.0", visible=False, test=test).with_requirement("boost/1.0")
        else:
            pkg.with_requirement("boost/1.0").with_requirement("gtest/1.0", visible=False, test=test)
        tc.save({"zlib/conanfile.py": GenConanfile("zlib"),
                 "gtest/conanfile.py": GenConanfile("gtest", "1.0").with_requirement("zlib/1.0"),
                 "boost/conanfile.py": GenConanfile("boost", "1.0").with_requirement("zlib/1.1"),
                 "pkg1/conanfile.py": pkg})
        tc.run("export zlib --version=1.0")
        tc.run("export zlib --version=1.1")
        tc.run("export gtest")
        tc.run("export boost")
        tc.run("export pkg1")
        tc.run("graph info --requires=zlib/1.1 --requires=pkg1/1.0 --format=html", assert_error=True,
               redirect_stdout="graph.html")
        # Check that the graph.html is generated
        assert os.path.exists(os.path.join(tc.current_folder, "graph.html"))
        if order:
            assert "ERROR: Version conflict: Conflict between zlib/1.1 and zlib/1.0" in tc.out
            assert "Conflict originates from pkg1/1.0" in tc.out
        else:
            assert "ERROR: Version conflict: Conflict between zlib/1.0 and zlib/1.1" in tc.out
            assert "Conflict originates from gtest/1.0" in tc.out

    def test_transitive_version_range_no_conflict(self, order, test):
        # if in the case above, we use a version-range, we can avoid the conflict
        tc = TestClient(light=True)
        pkg1 = GenConanfile("pkg1", "1.0")
        if order:
            pkg1.with_requirement("pkg3/[*]", visible=False, test=test).with_requirement("pkg2/1.0")
        else:
            pkg1.with_requirement("pkg2/1.0").with_requirement("pkg3/[*]", visible=False, test=test)
        tc.save({"pkg3/conanfile.py": GenConanfile("pkg3"),
                 "pkg2/conanfile.py": GenConanfile("pkg2", "1.0").with_requirement("pkg3/1.1"),
                 "pkg1/conanfile.py": pkg1})
        tc.run("export pkg3 --version=1.0")
        tc.run("export pkg3 --version=1.1")
        tc.run("export pkg2")
        # Creating this pkg1 does generate a conflict
        tc.run("export pkg1")
        tc.run("graph info --requires=pkg3/1.1 --requires=pkg1/1.0 --format=json")
        if not test:
            assert "WARN: risk: Packages required both with visible=True and visible=False" in tc.out
            assert "pkg3/1.1: Required by pkg1/1.0" in tc.out
        else:
            assert "WARN: risk" not in tc.out
        graph = json.loads(tc.stdout)
        assert len(graph["graph"]["nodes"]) == 4  # This was having an orphan node!!!
        pkg1 = graph["graph"]["nodes"]["2"]
        deps = pkg1["dependencies"]
        assert len(deps) == 2
        assert "pkg1/1.0" in pkg1["ref"]
        dep_pkg2 = deps["3"]
        dep_pkg3 = deps["1"]
        assert dep_pkg2["ref"] == "pkg2/1.0"
        assert dep_pkg2["visible"] is True
        assert dep_pkg3["ref"] == "pkg3/1.1"
        assert dep_pkg2["visible"] is True

    def test_transitive_version_range_no_conflict_second_level(self, order, test):
        # cli --------------------------------------------> zlib/1.1
        #   \--> pkg1/1.0 -(visible=False) -gtest---------> zlib/[*] (no conflict, range)
        #             \----> boost/1.0 -------------------> zlib/1.1 (no conflict)
        tc = TestClient(light=True)
        pkg = GenConanfile("pkg1", "1.0")
        if order:
            pkg.with_requirement("gtest/1.0", visible=False, test=test).with_requirement("boost/1.0")
        else:
            pkg.with_requirement("boost/1.0").with_requirement("gtest/1.0", visible=False, test=test)
        tc.save({"zlib/conanfile.py": GenConanfile("zlib"),
                 "gtest/conanfile.py": GenConanfile("gtest", "1.0").with_requirement("zlib/[*]"),
                 "boost/conanfile.py": GenConanfile("boost", "1.0").with_requirement("zlib/1.1"),
                 "pkg1/conanfile.py": pkg})
        tc.run("export zlib --version=1.0")
        tc.run("export zlib --version=1.1")
        tc.run("export gtest")
        tc.run("export boost")
        tc.run("export pkg1")
        tc.run("graph info --requires=zlib/1.1 --requires=pkg1/1.0 --build=missing")
        assert "pkg1/1.0: WARN: risk: This package has 2 different dependencies" not in tc.out
        # This doesn't conflict, but depending on the order, it is possible to have 2 differnet
        # dependency graphs. If gtest is evaluated first, there will be no diamond, and
        # gtest->zlib/1.1 will be private, and there will be another branch with another zlib/1.1
        # node as regular requires.
        # In both cases, it seems that zlib is not Skipped when it is necessary to build gtest

    def test_transitive_orphans(self, order, test):
        tc = TestClient(light=True)
        pkg1 = GenConanfile("pkg1", "1.0")
        if order:
            pkg1.with_requirement("pkg3/[*]", visible=False, test=test).with_requirement("pkg2/1.0")
        else:
            pkg1.with_requirement("pkg2/1.0").with_requirement("pkg3/[*]", visible=False, test=test)
        tc.save({"pkg4/conanfile.py": GenConanfile("pkg4", "0.1"),
                 "pkg3/conanfile.py": GenConanfile("pkg3").with_requires("pkg4/0.1"),
                 "pkg2/conanfile.py": GenConanfile("pkg2", "1.0").with_requirement("pkg3/1.1"),
                 "pkg1/conanfile.py": pkg1})
        tc.run("export pkg4")
        tc.run("export pkg3 --version=1.0")
        tc.run("export pkg3 --version=1.1")
        tc.run("export pkg2")
        # Creating this pkg1 does generate a conflict
        tc.run("export pkg1")

        tc.run("graph info --requires=pkg3/1.1 --requires=pkg1/1.0 --format=json")
        if not test:
            assert "WARN: risk: Packages required both with visible=True and visible=False" in tc.out
            assert "pkg3/1.1: Required by pkg1/1.0" in tc.out
        else:
            assert "WARN: risk" not in tc.out
        graph = json.loads(tc.stdout)
        assert len(graph["graph"]["nodes"]) == 5  # This was having an orphan node!!!
        pkg1 = graph["graph"]["nodes"]["3"]
        assert "pkg1/1.0" in pkg1["ref"]
        deps = pkg1["dependencies"]
        assert len(deps) == 3
        dep_pkg2 = deps["4"]
        dep_pkg3 = deps["1"]
        assert dep_pkg2["ref"] == "pkg2/1.0"
        assert dep_pkg2["visible"] is True
        assert dep_pkg3["ref"] == "pkg3/1.1"
        assert dep_pkg2["visible"] is True



import json
from collections import OrderedDict

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient, TestServer


@pytest.fixture
def create_libs():
    tc = TestClient(default_server_user=True, light=True)
    tc.save({"conanfile.py": GenConanfile()})
    tc.run("create . --name=zlib --version=1.0")
    tc.run("create . --name=zlib --version=2.0")
    tc.run("create . --name=foo --version=1.0")
    tc.run("create . --name=foo --version=2.0")

    tc.save({"conanfile.py": GenConanfile().with_requires("foo/[>=1.0]")})
    tc.run("create . --name=libcurl --version=1.0")

    # Upload all created libs to the remote
    tc.run("upload * -c -r=default")

    tc.save({"conanfile.py": GenConanfile("app", "1.0")
            .with_requires("zlib/1.0", "libcurl/[>=1.0]")})
    return tc


def test_outdated_command(create_libs):
    tc = create_libs

    # Remove versions from cache so they are found as outdated
    tc.run("remove foo/2.0 -c")
    tc.run("remove zlib/2.0 -c")

    tc.run("graph outdated . --format=json")
    output = json.loads(tc.stdout)
    assert "zlib" in output
    assert "foo" in output
    assert "libcurl" not in output
    assert output["zlib"]["current_versions"] == ["zlib/1.0"]
    assert output["zlib"]["version_ranges"] == []
    assert output["zlib"]["latest_remote"]["ref"] == "zlib/2.0"
    assert output["zlib"]["latest_remote"]["remote"].startswith("default")
    assert output["foo"]["current_versions"] == ["foo/1.0"]
    assert output["foo"]["version_ranges"] == ["foo/[>=1.0]"]
    assert output["foo"]["latest_remote"]["ref"] == "foo/2.0"
    assert output["foo"]["latest_remote"]["remote"].startswith("default")


def test_recipe_with_lockfile(create_libs):
    tc = create_libs

    # Remove versions from cache so they are found as outdated
    tc.run("remove foo/2.0 -c")
    tc.run("remove zlib/2.0 -c")

    # Creating the lockfile sets foo/1.0 as only valid version for the recipe
    tc.run("lock create .")
    tc.run("graph outdated . --format=json")
    output = json.loads(tc.stdout)
    assert "foo" in output
    assert output["foo"]["current_versions"] == ["foo/1.0"]
    # Creating the lockfile makes the previous range obsolete
    assert output["foo"]["version_ranges"] == []

    # Adding foo/2.0 to the lockfile forces the download so foo is no longer outdated
    tc.run("lock add --requires=foo/2.0")
    tc.run("graph outdated . --format=json")
    output = json.loads(tc.stdout)
    assert "foo" not in output


def test_recipe_with_no_remote_ref(create_libs):
    tc = create_libs

    # Remove versions from cache so they are found as outdated
    tc.run("remove foo/2.0 -c")
    tc.run("remove zlib/2.0 -c")
    tc.run("remove libcurl -c")

    tc.run("graph outdated . --format=json")
    output = json.loads(tc.stdout)
    # Check that libcurl doesn't appear as there is no version in the remotes
    assert "zlib" in output
    assert "foo" in output
    assert "libcurl" not in output


def test_cache_ref_newer_than_latest_in_remote(create_libs):
    tc = create_libs

    tc.run("remove foo/2.0 -c -r=default")

    tc.run("graph outdated . --format=json")
    output = json.loads(tc.stdout)
    # Check that foo doesn't appear because the version in cache is higher than the latest version
    # in remote
    assert "zlib" in output
    assert "libcurl" not in output
    assert "foo" not in output


def test_two_remotes():
    # remote1: zlib/1.0, libcurl/2.0, foo/1.0
    # remote2: zlib/[1.0, 2.0], libcurl/1.0, foo/1.0
    # local cache: zlib/1.0, libcurl/1.0, foo/1.0
    servers = OrderedDict()
    for i in [1, 2]:
        test_server = TestServer()
        servers["remote%d" % i] = test_server

    tc = TestClient(servers=servers, inputs=2 * ["admin", "password"], light=True)

    tc.save({"conanfile.py": GenConanfile()})
    tc.run("create . --name=zlib --version=1.0")
    tc.run("create . --name=foo --version=1.0")
    tc.run("create . --name=zlib --version=2.0")

    tc.save({"conanfile.py": GenConanfile().with_requires("foo/[>=1.0]")})
    tc.run("create . --name=libcurl --version=1.0")
    tc.run("create . --name=libcurl --version=2.0")

    # Upload the created libraries  1.0 to remotes
    tc.run("upload zlib/1.0 -c -r=remote1")
    tc.run("upload libcurl/2.0 -c -r=remote1")
    tc.run("upload foo/1.0 -c -r=remote1")

    tc.run("upload zlib/* -c -r=remote2")
    tc.run("upload libcurl/1.0 -c -r=remote2")
    tc.run("upload foo/1.0 -c -r=remote2")

    # Remove from cache the 2.0 libraries
    tc.run("remove libcurl/2.0 -c")
    tc.run("remove zlib/2.0 -c")

    tc.save({"conanfile.py": GenConanfile("app", "1.0")
            .with_requires("zlib/1.0", "libcurl/[>=1.0]")})

    tc.run("graph outdated . --format=json")
    output = json.loads(tc.stdout)
    assert "zlib" in output
    assert "libcurl" in output
    assert "foo" not in output
    assert output["libcurl"]["latest_remote"]["ref"].startswith("libcurl/2.0")
    assert output["libcurl"]["latest_remote"]["remote"].startswith("remote1")
    assert output["zlib"]["latest_remote"]["ref"].startswith("zlib/2.0")
    assert output["zlib"]["latest_remote"]["remote"].startswith("remote2")


def test_duplicated_tool_requires():
    tc = TestClient(default_server_user=True)
    # Create libraries needed to generate the dependency graph
    tc.save({"conanfile.py": GenConanfile()})
    tc.run("create . --name=cmake --version=1.0")
    tc.run("create . --name=cmake --version=2.0")
    tc.run("create . --name=cmake --version=3.0")
    tc.save({"conanfile.py": GenConanfile().with_tool_requires("cmake/1.0")})
    tc.run("create . --name=foo --version=1.0")
    tc.save({"conanfile.py": GenConanfile().with_tool_requires("cmake/[>=1.0]")})
    tc.run("create . --name=bar --version=1.0")

    # Upload the created libraries to remote
    tc.run("upload * -c -r=default")

    tc.save({"conanfile.py": GenConanfile("app", "1.0")
            .with_requires("foo/1.0", "bar/1.0").with_tool_requires("cmake/[<=2.0]")})

    tc.run("graph outdated . --format=json")
    output = json.loads(tc.stdout)

    assert sorted(output["cmake"]["current_versions"]) == ["cmake/1.0", "cmake/2.0", "cmake/3.0"]
    assert sorted(output["cmake"]["version_ranges"]) == ["cmake/[<=2.0]", "cmake/[>=1.0]"]
    assert output["cmake"]["latest_remote"]["ref"] == "cmake/3.0"


def test_no_outdated_dependencies():
    tc = TestClient(default_server_user=True)

    tc.save({"conanfile.py": GenConanfile()})
    tc.run("create . --name=foo --version=1.0")
    tc.run("upload * -c -r=default")

    tc.run("graph outdated .")

    assert "No outdated dependencies in graph" in tc.stdout



import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


def test_plugin_cmd_wrapper():
    c = TestClient()
    plugins = os.path.join(c.cache_folder, "extensions", "plugins")
    wrapper = textwrap.dedent("""
        def cmd_wrapper(cmd, **kwargs):
            return 'echo "{}"'.format(cmd)
        """)
    save(os.path.join(plugins, "cmd_wrapper.py"), wrapper)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def generate(self):
                self.run("Hello world")
                self.run("Other stuff")
        """)
    c.save({"conanfile.py": conanfile})
    c.run("install .")
    assert 'Hello world' in c.out
    assert 'Other stuff' in c.out


def test_plugin_cmd_wrapper_conanfile():
    """
    we can get the name of the caller conanfile too
    """
    c = TestClient()
    plugins = os.path.join(c.cache_folder, "extensions", "plugins")
    wrapper = textwrap.dedent("""
        def cmd_wrapper(cmd, conanfile, **kwargs):
            return 'echo "{}!:{}!"'.format(conanfile.ref, cmd)
        """)
    save(os.path.join(plugins, "cmd_wrapper.py"), wrapper)
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            def generate(self):
                self.run("Hello world")
        """)
    c.save({"conanfile.py": conanfile})
    c.run("install . --name=pkg --version=0.1")
    assert 'pkg/0.1!:Hello world!' in c.out


def test_plugin_profile_error_vs():
    c = TestClient()
    c.save({"conanfile.py": GenConanfile("pkg", "1.0")})
    c.run("create . -s compiler=msvc -s compiler.version=15 -s compiler.cppstd=14",
          assert_error=True)
    assert "The provided compiler.cppstd=14 is not supported by msvc 15. Supported values are: []" in c.out
    c.run("create . -s compiler=msvc -s compiler.version=170 -s compiler.cppstd=14",
          assert_error=True)
    assert "The provided compiler.cppstd=14 is not supported by msvc 170. Supported values are: []" in c.out
    c.run("create . -s compiler=msvc -s compiler.version=190 -s compiler.cppstd=14")
    assert "Installing packages" in c.out


def test_plugin_profile_error_vscstd():
    c = TestClient()
    c.save({"conanfile.py": GenConanfile("pkg", "1.0").with_class_attribute("languages = 'C'")})
    c.run("create . -s compiler=msvc -s compiler.version=190 -s compiler.cstd=23", assert_error=True)
    assert "The provided compiler.cstd=23 is not supported by msvc 190. Supported values are: []" in c.out
    c.run("create . -s compiler=msvc -s compiler.version=193 -s compiler.cstd=23", assert_error=True)
    assert "The provided compiler.cstd=23 is not supported by msvc 193. Supported values are: ['11', '17']" in c.out
    c.run("create . -s compiler=msvc -s compiler.version=193 -s compiler.cstd=17")
    assert "Installing packages" in c.out



import json
import os

import pytest

from conan.api.model import PkgReference
from conan.internal.util.files import save
from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient

PKG_ID = "da39a3ee5e6b4b0d3255bfef95601890afd80709"
RREV = "a9ec2e5fbb166568d4670a9cd1ef4b26"


@pytest.fixture(scope="module")
def client():
    tc = TestClient(light=True, default_server_user=True)
    tc.save({"conanfile.py": GenConanfile("pkg")})
    tc.run("create . --version=1.0")
    assert RREV == tc.exported_recipe_revision()
    assert PKG_ID == tc.created_package_id("pkg/1.0")
    tc.run("upload * -r=default -c")
    tc.run("remove * -c")
    tc.run("remote update default --recipes-only")
    return tc


def test_graph_binary_analyze(client):
    client.run("install --requires=pkg/1.0 -f=json", assert_error=True)
    client.assert_listed_binary({"pkg/1.0": (PKG_ID, "Missing")})

    client.run("graph explain --requires=pkg/1.0")
    assert "ERROR: No package binaries exist" in client.out


def test_list_packages(client):
    client.run("list pkg/1.0:* -r=default -f=json", redirect_stdout="list.json")
    result = json.loads(client.load("list.json"))
    assert len(result["default"]["pkg/1.0"]["revisions"][RREV]["packages"]) == 0

    client.run(f"list pkg/1.0:{PKG_ID} -r=default -f=json", redirect_stdout="list.json")
    result = json.loads(client.load("list.json"))
    assert "error" in result["default"]  # not found error


def test_download_package(client):
    client.run("download pkg/1.0:* -r=default -f=json", redirect_stdout="download.json")
    result = json.loads(client.load("download.json"))
    assert len(result["Local Cache"]["pkg/1.0"]["revisions"][RREV]["packages"]) == 0


def test_remove_package(client):
    client.run(f"remove pkg/1.0:{PKG_ID} -r=default -c -f=json", redirect_stdout="remove.json")
    result = json.loads(client.load("remove.json"))
    assert result["default"] == {}

    client.run(f"remove pkg/1.0:* -r=default -c -f=json", redirect_stdout="remove.json")
    result = json.loads(client.load("remove.json"))
    assert result["default"] == {}

    client.run(f"remove pkg/1.0 -r=default -c -f=json", redirect_stdout="remove.json")
    result = json.loads(client.load("remove.json"))
    assert "packages" not in result["default"]["pkg/1.0"]["revisions"][RREV]


@pytest.mark.parametrize("pattern", ["*", "pkg/2.0", "pkg/2.0:*", f"pkg/2.0:{PKG_ID}"])
def test_upload_still_works(client, pattern):
    client.run("create . --version=2.0")
    layout = client.created_layout()
    save(os.path.join(layout.metadata(), "metadata.log"), "New hello")

    client.run(f"upload {pattern} -r=default -c")

    # Now check that the binary was in fact uploaded
    client.run("remote update default --recipes-only=False")
    client.run("remove * -c")
    client.run("list pkg/2.0:* -r=default -f=json", redirect_stdout="list.json")
    result = json.loads(client.load("list.json"))
    assert len(result["default"]["pkg/2.0"]["revisions"][RREV]["packages"]) == 1
    assert PKG_ID in result["default"]["pkg/2.0"]["revisions"][RREV]["packages"]

    client.run(f"download {pattern} -r=default -f=json --metadata=*", redirect_stdout="download.json")
    pkg2_layout = client.get_latest_pkg_layout(PkgReference().loads(f"pkg/2.0#{RREV}:{PKG_ID}"))
    assert os.path.exists(os.path.join(pkg2_layout.metadata(), "metadata.log"))



import platform

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_library_order():
    #
    # project -> sdl2_ttf -> freetype ------------> bzip2
    #             \             \----->libpng  -> zlib
    #              \------> sdl2----------------->/
    # Order: sdl2_ttf freetype sdl2 libpng zlib bzip

    c = TestClient()

    def _export(libname, refs=None):
        conanfile = GenConanfile(libname, "0.1").with_package_info(cpp_info={"libs": [libname]},
                                                                   env_info={})
        for r in refs or []:
            conanfile.with_requires(f"{r}/0.1")
        c.save({"conanfile.py": conanfile}, clean_first=True)
        c.run("export .")

    _export("zlib")
    _export("bzip2")
    _export("sdl2", ["zlib"])
    _export("libpng", ["zlib"])
    _export("freetype", ["bzip2", "libpng"])
    _export("sdl2_ttf", ["freetype", "sdl2"])
    _export("app", ["sdl2_ttf"])

    c.run("install . --build missing -g AutotoolsDeps")

    deps = "conanautotoolsdeps.bat" if platform.system() == "Windows" else "conanautotoolsdeps.sh"
    autotoolsdeps = c.load(deps)
    assert '-lsdl2_ttf -lfreetype -lsdl2 -llibpng -lzlib -lbzip2' in autotoolsdeps


# TODO: Add a test that manages too system libs



from bottle import Bottle

from conan.internal.errors import EXCEPTION_CODE_MAPPING
from conans.server.rest.bottle_plugins.http_basic_authentication import HttpBasicAuthentication
from conans.server.rest.bottle_plugins.jwt_authentication import JWTAuthentication
from conans.server.rest.bottle_plugins.return_handler import ReturnHandlerPlugin
from conans.server.rest.controller.v2.ping import PingController
from conans.server.rest.controller.v2.users import UsersController
from conans.server.rest.controller.v2.conan import ConanControllerV2
from conans.server.rest.controller.v2.delete import DeleteControllerV2
from conans.server.rest.controller.v2.revisions import RevisionsController
from conans.server.rest.controller.v2.search import SearchControllerV2


class ApiV2(Bottle):

    def __init__(self, credentials_manager, server_capabilities):

        self.credentials_manager = credentials_manager
        self.server_capabilities = server_capabilities
        Bottle.__init__(self)

    def setup(self):
        self.install_plugins()

        # Capabilities in a ping
        PingController().attach_to(self)

        SearchControllerV2().attach_to(self)
        DeleteControllerV2().attach_to(self)
        ConanControllerV2().attach_to(self)
        RevisionsController().attach_to(self)

        # Install users controller
        UsersController().attach_to(self)

    def install_plugins(self):
        # Second, check Http Basic Auth
        self.install(HttpBasicAuthentication())

        # Map exceptions to http return codes
        self.install(ReturnHandlerPlugin(EXCEPTION_CODE_MAPPING))

        # Handle jwt auth
        self.install(JWTAuthentication(self.credentials_manager))



import os.path
import re

import pytest

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.test_files import temp_folder
from conan.test.utils.tools import TestClient
from conan.internal.util.files import save


@pytest.mark.parametrize("use_pkglist", [True, False])
def test_cache_clean(use_pkglist):
    c = TestClient(default_server_user=True, light=True)
    c.save({"conanfile.py": GenConanfile("pkg", "0.1").with_exports("*").with_exports_sources("*"),
            "sorces/file.txt": ""})
    c.run("create .")
    ref_layout = c.exported_layout()
    pkg_layout = c.created_layout()
    c.run("upload * -c -r=default")  # Force creation of tgzs
    assert os.path.exists(ref_layout.source())
    assert os.path.exists(ref_layout.download_export())
    assert os.path.exists(pkg_layout.build())
    assert os.path.exists(pkg_layout.download_package())

    if use_pkglist:
        c.run("list *:*#* -f=json", redirect_stdout="pkglist.json")
    arg = "--list=pkglist.json" if use_pkglist else "*"
    c.run(f'cache clean {arg} -s -b -v')
    assert f"{ref_layout.reference.repr_notime()}: Cleaning recipe cache contents" in c.out
    assert f"{pkg_layout.reference}: Cleaning package cache contents" in c.out
    assert not os.path.exists(pkg_layout.build())
    assert not os.path.exists(ref_layout.source())
    assert os.path.exists(ref_layout.download_export())
    assert os.path.exists(pkg_layout.download_package())

    c.run('cache clean -d')
    assert not os.path.exists(ref_layout.download_export())
    assert not os.path.exists(pkg_layout.download_package())

    c.run("cache clean -bs -v")
    assert "Cleaning 0 backup sources" in c.out


def test_cache_clean_all():
    c = TestClient(light=True)
    c.save({"pkg1/conanfile.py": GenConanfile("pkg", "0.1").with_class_attribute("revision_mode='scm'"),
            "pkg2/conanfile.py": GenConanfile("pkg", "0.2").with_package("error"),
            "pkg3/conanfile.py": GenConanfile("pkg", "0.3")})
    c.run("create pkg1", assert_error=True)
    c.run("create pkg2", assert_error=True)
    c.run("create pkg3")
    pref = c.created_package_reference("pkg/0.3")
    tmp_folder = os.path.join(c.cache_folder, "p", "t")
    assert len(os.listdir(tmp_folder)) == 1  # Failed export was here
    builds_folder = os.path.join(c.cache_folder, "p", "b")
    assert len(os.listdir(builds_folder)) == 2  # both builds are here
    c.run('cache clean')
    assert not os.path.exists(tmp_folder)
    assert len(os.listdir(builds_folder)) == 1  # only correct pkg/0.3 remains
    # Check correct package removed all
    ref_layout = c.get_latest_ref_layout(pref.ref)
    pkg_layout = c.get_latest_pkg_layout(pref)
    assert not os.path.exists(ref_layout.source())
    assert not os.path.exists(ref_layout.download_export())
    assert not os.path.exists(pkg_layout.build())
    assert not os.path.exists(pkg_layout.download_package())

    # A second clean like this used to crash
    # as it tried to delete a folder that was not there and tripped shutils up
    c.run('cache clean')
    assert not os.path.exists(tmp_folder)


def test_cache_multiple_builds_same_prev_clean():
    """
    Different consecutive builds will create exactly the same folder, for the
    same exact prev, not leaving trailing non-referenced folders
    """
    c = TestClient(light=True)
    c.save({"conanfile.py": GenConanfile("pkg", "0.1")})
    c.run("create .")
    create_out = c.out
    c.run("cache path pkg/0.1:da39a3ee5e6b4b0d3255bfef95601890afd80709")
    path1 = str(c.stdout)
    assert path1 in create_out
    c.run("create .")
    create_out = c.out
    c.run("cache path pkg/0.1:da39a3ee5e6b4b0d3255bfef95601890afd80709")
    path2 = str(c.stdout)
    assert path2 in create_out
    assert path1 == path2

    builds_folder = os.path.join(c.cache_folder, "p", "b")
    assert len(os.listdir(builds_folder)) == 1  # only one build
    c.run('cache clean')
    assert len(os.listdir(builds_folder)) == 1  # one build not cleaned
    c.run('remove * -c')
    assert len(os.listdir(builds_folder)) == 0  # no folder remain


def test_cache_multiple_builds_diff_prev_clean():
    """
    Different consecutive builds will create different folders, even if for the
    same exact prev, leaving trailing non-referenced folders
    """
    c = TestClient(light=True)
    package_lines = 'save(self, os.path.join(self.package_folder, "foo.txt"), str(time.time()))'
    gen = GenConanfile("pkg", "0.1").with_package(package_lines).with_import("import os, time") \
                                    .with_import("from conan.tools.files import save")
    c.save({"conanfile.py": gen})
    c.run("create .")
    create_out = c.out
    c.run("cache path pkg/0.1:da39a3ee5e6b4b0d3255bfef95601890afd80709")
    path1 = str(c.stdout)
    assert path1 in create_out
    c.run("create .")
    create_out = c.out
    c.run("cache path pkg/0.1:da39a3ee5e6b4b0d3255bfef95601890afd80709")
    path2 = str(c.stdout)
    assert path2 in create_out
    assert path1 != path2

    builds_folder = os.path.join(c.cache_folder, "p", "b")
    assert len(os.listdir(builds_folder)) == 2  # both builds are here
    c.run('cache clean')
    assert len(os.listdir(builds_folder)) == 2  # two builds will remain, both are valid
    c.run('remove * -c')
    assert len(os.listdir(builds_folder)) == 0  # no folder remain


def test_cache_clean_custom_storage():
    c = TestClient()
    t = temp_folder(path_with_spaces=False)
    save(c.paths.global_conf_path, f"core.cache:storage_path={t}")
    c.save({"conanfile.py": GenConanfile("pkg", "0.1").with_cmake_build()})
    c.run("create .", assert_error=True)
    build_folder = re.search(r"pkg/0.1: Building your package in (\S+)", str(c.out)).group(1)
    assert os.listdir(build_folder)
    # now clean
    c.run("cache clean")
    assert not os.path.exists(build_folder)



import platform
import textwrap

import pytest

from conan.test.assets.sources import gen_function_cpp
from test.functional.utils import check_exe_run
from conan.test.utils.tools import TestClient


@pytest.mark.parametrize(
    "compiler, version, runtime, cppstd, build_type, defines, cflags, cxxflags, sharedlinkflags, exelinkflags",
    [
        ("msvc", "191", "dynamic", "14", "Release", [], [], [], [], []),
        ("msvc", "191", "dynamic", "14", "Release",
         ["TEST_DEFINITION1", "TEST_DEFINITION2=0", "TEST_DEFINITION3=", "TEST_DEFINITION4=TestPpdValue4",
          "TEST_DEFINITION5=__declspec(dllexport)", "TEST_DEFINITION6=foo bar"],
         ["/GL"], ["/GL"], ["/LTCG"], ["/LTCG"]),
        ("msvc", "191", "static", "17", "Debug", [], [], [], [], []),
    ],
)
@pytest.mark.skipif(platform.system() != "Windows", reason="Only for windows")
def test_toolchain_nmake(compiler, version, runtime, cppstd, build_type,
                         defines, cflags, cxxflags, sharedlinkflags, exelinkflags):
    client = TestClient(path_with_spaces=False)
    settings = {"compiler": compiler,
                "compiler.version": version,
                "compiler.cppstd": cppstd,
                "compiler.runtime": runtime,
                "build_type": build_type,
                "arch": "x86_64"}

    serialize_array = lambda arr: "[{}]".format(",".join([f"'{v}'" for v in arr]))
    conf = {
        "tools.build:defines": serialize_array(defines) if defines else "",
        "tools.build:cflags": serialize_array(cflags) if cflags else "",
        "tools.build:cxxflags": serialize_array(cxxflags) if cxxflags else "",
        "tools.build:sharedlinkflags": serialize_array(sharedlinkflags) if sharedlinkflags else "",
        "tools.build:exelinkflags": serialize_array(exelinkflags) if exelinkflags else "",
    }

    # Build the profile according to the settings provided
    settings = " ".join('-s %s="%s"' % (k, v) for k, v in settings.items() if v)

    client.run("new cmake_lib -d name=dep -d version=1.0")
    conf = " ".join(f'-c {k}="{v}"' for k, v in conf.items() if v)
    client.run(f'create . -tf=\"\" {settings} {conf}'
               f' -c tools.cmake.cmaketoolchain:generator="Visual Studio 15"')

    # Rearrange defines to macro / value dict
    conf_preprocessors = {}
    for define in defines:
        if "=" in define:
            key, value = define.split("=", 1)
            # gen_function_cpp doesn't properly handle empty macros
            if value:
                conf_preprocessors[key] = value
        else:
            conf_preprocessors[define] = "1"

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            requires = "dep/1.0"
            generators = "NMakeToolchain", "NMakeDeps"

            def build(self):
                self.run(f"nmake /f makefile")
            """)
    makefile = textwrap.dedent("""\
        all: simple.exe

        .cpp.obj:
          $(CPP) $*.cpp

        simple.exe: simple.obj
          $(CPP) simple.obj
        """)
    client.save({"conanfile.py": conanfile,
                 "makefile": makefile,
                 "simple.cpp": gen_function_cpp(name="main", includes=["dep"], calls=["dep"], preprocessor=conf_preprocessors.keys())},
                clean_first=True)
    client.run(f"build . {settings} {conf}")
    client.run_command("simple.exe")
    assert "dep/1.0" in client.out
    check_exe_run(client.out, "main", "msvc", version, build_type, "x86_64", cppstd, conf_preprocessors)


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires Windows")
@pytest.mark.tool("clang", "18")
def test_toolchain_nmake_clang():
    compiler = "clang"
    version = "18"
    runtime = "dynamic"
    cppstd = "14"
    build_type = "Debug"
    defines = ["TEST_DEFINITION1", "TEST_DEFINITION2=0", "TEST_DEFINITION3=", "TEST_DEFINITION4=TestPpdValue4",
          "TEST_DEFINITION5=__declspec(dllexport)", "TEST_DEFINITION6=foo bar"]
    cflags = cxxflags = sharedlinkflags = exelinkflags = []
    client = TestClient(path_with_spaces=False)
    settings = {"compiler": compiler,
                "compiler.version": version,
                "compiler.cppstd": cppstd,
                "compiler.runtime": runtime,
                "build_type": build_type,
                "compiler.runtime_version": "v144",
                "arch": "x86_64"}

    serialize_array = lambda arr: "[{}]".format(",".join([f"'{v}'" for v in arr]))
    conf = {
        "tools.build:defines": serialize_array(defines) if defines else "",
        "tools.build:cflags": serialize_array(cflags) if cflags else "",
        "tools.build:cxxflags": serialize_array(cxxflags) if cxxflags else "",
        "tools.build:sharedlinkflags": serialize_array(sharedlinkflags) if sharedlinkflags else "",
        "tools.build:exelinkflags": serialize_array(exelinkflags) if exelinkflags else "",
        "tools.build:compiler_executables": r'{\"c\": \"clang-cl\", \"cpp\": \"clang-cl\"}',
        "tools.cmake.cmaketoolchain:generator": "Visual Studio 17",
    }

    # Build the profile according to the settings provided
    settings = " ".join('-s %s="%s"' % (k, v) for k, v in settings.items() if v)

    client.run("new cmake_lib -d name=dep -d version=1.0")
    conf = " ".join(f'-c {k}="{v}"' for k, v in conf.items() if v)
    client.run(f'create . -tf=\"\" {settings} {conf}')

    # Rearrange defines to macro / value dict
    conf_preprocessors = {}
    for define in defines:
        if "=" in define:
            key, value = define.split("=", 1)
            # gen_function_cpp doesn't properly handle empty macros
            if value:
                conf_preprocessors[key] = value
        else:
            conf_preprocessors[define] = "1"

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            requires = "dep/1.0"
            generators = "NMakeToolchain", "NMakeDeps"

            def build(self):
                self.run(f"nmake /f makefile")
            """)
    makefile = textwrap.dedent("""\
        all: simple.exe

        simple.exe: simple.cpp
          $(CPP) simple.cpp -o simple.exe
        """)
    client.save({"conanfile.py": conanfile,
                 "makefile": makefile,
                 "simple.cpp": gen_function_cpp(name="main", includes=["dep"], calls=["dep"], preprocessor=conf_preprocessors.keys())},
                clean_first=True)
    client.run(f"build . {settings} {conf}")
    client.run_command("simple.exe")
    assert "dep/1.0" in client.out
    check_exe_run(client.out, "main", "clang", "19.1", build_type, "x86_64", cppstd, conf_preprocessors)



import os
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_editable_folders_root():
    """ Editables with self.folders.root = ".." should work too
    """
    c = TestClient()
    pkg = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def layout(self):
                self.folders.root = ".."
                self.folders.build = "mybuild"
                self.cpp.build.libdirs = ["mylibdir"]

            def generate(self):
                self.output.info(f"PKG source_folder {self.source_folder}")

            def package_info(self):
                self.output.info(f"PKG package_folder {self.package_folder}")
        """)
    app = textwrap.dedent("""
        from conan import ConanFile

        class App(ConanFile):
            requires = "pkg/0.1"
            def generate(self):
                pkg_libdir = self.dependencies["pkg"].cpp_info.libdir
                self.output.info(f"PKG LIBDIR {pkg_libdir}")
        """)
    c.save({"libs/pkg/conanfile.py": pkg,
            "conanfile.py": app})

    c.run("editable add libs/pkg")
    c.run("install . --build=editable")
    libdir = os.path.join(c.current_folder, "libs", "mybuild", "mylibdir")
    assert f"conanfile.py: PKG LIBDIR {libdir}" in c.out
    assert f"pkg/0.1: PKG package_folder {c.current_folder}" in c.out
    assert f"pkg/0.1: PKG source_folder {c.current_folder}" in c.out


def test_editable_folders_sibiling_root():
    """ Editables with self.folders.root = ".." should work too for sibling folders
    """
    c = TestClient()
    pkg = textwrap.dedent("""
        from conan import ConanFile

        class Pkg(ConanFile):
            name = "pkg"
            version = "0.1"

            def layout(self):
                self.folders.root = ".."
                self.folders.build = "mybuild"
                self.cpp.build.libdirs = ["mylibdir"]

            def generate(self):
                self.output.info(f"PKG source_folder {self.source_folder}")

            def package_info(self):
                self.output.info(f"PKG package_folder {self.package_folder}")
        """)
    app = textwrap.dedent("""
        from conan import ConanFile

        class App(ConanFile):
            requires = "pkg/0.1"
            def generate(self):
                pkg_libdir = self.dependencies["pkg"].cpp_info.libdir
                self.output.info(f"PKG LIBDIR {pkg_libdir}")
        """)
    c.save({"pkg/conanfile.py": pkg,
            "app/conanfile.py": app})

    c.run("editable add pkg")
    c.run("install app --build=editable")
    libdir = os.path.join(c.current_folder, "mybuild", "mylibdir")
    assert f"conanfile.py: PKG LIBDIR {libdir}" in c.out
    assert f"pkg/0.1: PKG source_folder {c.current_folder}" in c.out
    assert f"pkg/0.1: PKG package_folder {c.current_folder}" in c.out


def test_editable_matching_folder_names():
    # https://github.com/conan-io/conan/issues/14091
    client = TestClient()
    client.save({"hello-say/conanfile.py": GenConanfile("say", "0.1"),
                 "hello/conanfile.py": GenConanfile("hello", "0.1").with_settings("build_type")
                                                                   .with_require("say/0.1")})
    client.run("editable add hello-say")
    client.run("build hello-say")
    client.run("install hello -g CMakeDeps -s:b os=Linux")
    data = client.load("hello/say-release-data.cmake")
    hello_say_folder = os.path.join(client.current_folder, "hello-say").replace("\\", "/")
    assert f'set(say_PACKAGE_FOLDER_RELEASE "{hello_say_folder}")' in data
    env = client.load("hello/conanbuildenv-release.sh")
    assert "$script_folder/deactivate_conanbuildenv-release.sh" in env.replace("\\", "/")


def test_install_editable_build_folder_vars():
    """ Make sure that the consumer generates the editable files following build_folder_vars
    """
    c = TestClient()
    profile = textwrap.dedent("""
        [settings]
        os=Windows
        arch=x86_64
        build_type=Release
        [conf]
        tools.cmake.cmake_layout:build_folder_vars=['settings.os', 'settings.arch']
        """)
    lib = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout
        class Lib(ConanFile):
            name = "lib"
            version = "0.1"
            settings = "os", "arch", "build_type"
            generators = "CMakeToolchain"

            def layout(self):
                cmake_layout(self)
            """)
    app = textwrap.dedent("""
        from conan import ConanFile
        from conan.tools.cmake import cmake_layout
        class App(ConanFile):
            settings = "os", "arch", "build_type"
            requires = "lib/0.1"
            generators = "CMakeToolchain"

            def layout(self):
                cmake_layout(self)
            """)
    c.save({"lib/conanfile.py": lib,
            "app/conanfile.py": app,
            "profile": profile})
    c.run("editable add lib")
    c.run("build lib -pr=profile")
    c.run("install app -pr=profile --build=editable")
    path = os.path.join(c.current_folder, "lib", "build", "windows-x86_64", "Release", "generators")
    assert f"lib/0.1: Writing generators to {path}" in c.out



import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_lockfile_compatibility():
    c = TestClient()
    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "pkg"
            version = "1.0"
            settings = "build_type"
            def compatibility(self):
                if self.settings.build_type == "Release":
                    return [ {"settings": [("build_type", None)]}, ]
            """)
    c.save({"conanfile.py": conanfile,
            "profile": ""})
    c.run("create . -pr=profile")
    c.save({"conanfile.py": GenConanfile().with_requires("pkg/1.0")})
    c.run("install .")
    assert "pkg/1.0: Main binary package 'efa83b160a55b033c4ea706ddb980cd708e3ba1b' missing" in c.out
    assert "Found compatible package 'da39a3ee5e6b4b0d3255bfef95601890afd80709'" in c.out

    c.run("lock create conanfile.py")
    assert "pkg/1.0: Main binary package 'efa83b160a55b033c4ea706ddb980cd708e3ba1b' missing" in c.out
    assert "Found compatible package 'da39a3ee5e6b4b0d3255bfef95601890afd80709'" in c.out

    c.run("lock create conanfile.py --lockfile=conan.lock")
    assert "pkg/1.0: Main binary package 'efa83b160a55b033c4ea706ddb980cd708e3ba1b' missing" in c.out
    assert "Found compatible package 'da39a3ee5e6b4b0d3255bfef95601890afd80709'" in c.out

    c.run("install . --lockfile=conan.lock")
    c.assert_listed_binary({"pkg/1.0": ("da39a3ee5e6b4b0d3255bfef95601890afd80709", "Cache")})



import re
import textwrap

from conan.test.assets.genconanfile import GenConanfile
from conan.test.utils.tools import TestClient


def test_repackage():
    # consumer -> repackager -> liba
    #                  \------> libb
    client = TestClient(light=True)
    client.save({"conanfile.py": GenConanfile().with_package_file("liba.txt", "HelloA!")})
    client.run("create . --name=liba --version=1.0")
    client.save({"conanfile.py": GenConanfile().with_package_file("libb.txt", "HelloB!")})
    client.run("create . --name=libb --version=1.0 ")

    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import copy
        class Pkg(ConanFile):
            settings = "os"

            def requirements(self):
                self.requires("liba/1.0", headers=True, libs=True, visible=False, run=False)
                self.requires("libb/1.0", headers=True, libs=True, visible=False, run=False)

            def package(self):
                for r, d in self.dependencies.items():
                    copy(self, "*", src=d.package_folder,
                         dst=os.path.join(self.package_folder, r.ref.name))
        """)
    client.save({"conanfile.py": conanfile})
    client.run("create . --name=repackager --version=1.0")

    client.save({"conanfile.py": GenConanfile().with_requires("repackager/1.0")}, clean_first=True)
    client.run("install .")
    assert re.search(r"Skipped binaries(\s*)liba/1.0, libb/1.0", client.out)
    assert "repackager/1.0: Already installed!" in client.out


def test_repackage_library_self():
    c = TestClient()
    c.save({"conanfile.py": GenConanfile("liba", "1.0").with_package_file("a.txt", "A1.0!")})
    c.run("create .")

    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import copy
        class Pkg(ConanFile):
            name = "liba"
            version = "2.0"

            def requirements(self):
                self.requires("liba/1.0", visible=False)

            def package(self):
                for _, dep in self.dependencies.items():
                    copy(self, "*", src=dep.package_folder, dst=self.package_folder)
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")

    c.run("install --requires=liba/2.0 --deployer=full_deploy")
    assert re.search(r"Skipped binaries(\s*)liba/1.0", c.out)
    assert "liba/2.0: Already installed!" in c.out
    assert c.load("full_deploy/host/liba/2.0/a.txt") == "A1.0!"


def test_repackage_library_self_infinite_loop():
    c = TestClient()

    conanfile = textwrap.dedent("""
        from conan import ConanFile
        class Pkg(ConanFile):
            name = "liba"
            version = "1.0"

            def requirements(self):
                self.requires("liba/1.0", visible=False, headers=False, libs=False, run=False)
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create .", assert_error=True)
    assert "There is a cycle/loop in the graph" in c.out


def test_repackage_library_self_multiple():
    c = TestClient()
    c.save({"1/conanfile.py": GenConanfile("liba", "1.0").with_package_file("a1.txt", "A1.0!"),
            "2/conanfile.py": GenConanfile("liba", "2.0").with_package_file("a2.txt", "A2.0!")})
    c.run("create 1")
    c.run("create 2")

    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import copy
        class Pkg(ConanFile):
            name = "liba"
            version = "3.0"

            def requirements(self):
                # To avoid conflict among liba versions
                self.requires("liba/1.0", headers=False, libs=False, visible=False)
                self.requires("liba/2.0", visible=False)

            def package(self):
                for _, dep in self.dependencies.items():
                    copy(self, "*", src=dep.package_folder, dst=self.package_folder)
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")

    c.run("install --requires=liba/3.0 --deployer=full_deploy")
    assert re.search(r"Skipped binaries(\s*)liba/1.0, liba/2.0", c.out)
    assert "liba/3.0: Already installed!" in c.out
    assert c.load("full_deploy/host/liba/3.0/a1.txt") == "A1.0!"
    assert c.load("full_deploy/host/liba/3.0/a2.txt") == "A2.0!"


def test_repackage_library_self_transitive():
    c = TestClient()
    c.save({"a/conanfile.py": GenConanfile("liba", "1.0").with_package_file("a1.txt", "A1.0!"),
            "b/conanfile.py": GenConanfile("libb", "1.0").with_requires("liba/1.0")})
    c.run("create a")
    c.run("create b")

    conanfile = textwrap.dedent("""
        import os
        from conan import ConanFile
        from conan.tools.files import copy
        class Pkg(ConanFile):
            name = "liba"
            version = "3.0"

            def requirements(self):
                # libs=True, otherwise transitive liba is skipped
                self.requires("libb/1.0", headers=False, libs=True, visible=False, run=False)

            def package(self):
                for _, dep in self.dependencies.items():
                    copy(self, "*", src=dep.package_folder, dst=self.package_folder)
        """)
    c.save({"conanfile.py": conanfile})
    c.run("create .")

    c.run("install --requires=liba/3.0 --deployer=full_deploy")
    assert re.search(r"Skipped binaries(\s*)liba/1.0, libb/1.0", c.out)
    assert "liba/3.0: Already installed!" in c.out
    assert c.load("full_deploy/host/liba/3.0/a1.txt") == "A1.0!"



from unittest import mock
from unittest.mock import patch
import pytest

from conan.internal.api.detect.detect_api import _cc_compiler, detect_suncc_compiler, \
    detect_intel_compiler
from conan.internal.api.profile.detect import detect_defaults_settings
from conan.internal.model.version import Version
from conan.test.utils.mocks import RedirectedTestOutput
from conan.test.utils.tools import redirect_output
from conan.test.utils.env import environment_update


class TestDetect:
    @mock.patch("platform.machine", return_value="")
    def test_detect_empty_arch(self, _):
        result = detect_defaults_settings()
        result = dict(result)
        assert "arch" not in result

    @pytest.mark.parametrize("processor,bitness,version,expected_arch", [
        ['powerpc', '64', '7.1.0.0', 'ppc64'],
        ['powerpc', '32', '7.1.0.0', 'ppc32'],
        ['rs6000', None, '4.2.1.0', 'ppc32']
    ])
    def test_detect_aix(self, processor, bitness, version, expected_arch):
        with mock.patch("platform.machine", mock.MagicMock(return_value='XXXXXXXXXXXX')), \
                mock.patch("platform.processor", mock.MagicMock(return_value=processor)), \
                mock.patch("platform.system", mock.MagicMock(return_value='AIX')), \
                mock.patch("conan.internal.api.detect.detect_api._get_aix_conf",
                           mock.MagicMock(return_value=bitness)), \
                mock.patch('subprocess.check_output', mock.MagicMock(return_value=version)):
            result = detect_defaults_settings()
            result = dict(result)
            assert "AIX" == result['os']
            assert expected_arch == result['arch']

    @pytest.mark.parametrize("machine,expected_arch", [
        ['arm64', 'armv8'],
        ['i386', 'x86'],
        ['i686', 'x86'],
        ['i86pc', 'x86'],
        ['amd64', 'x86_64'],
        ['aarch64', 'armv8'],
        ['sun4v', 'sparc']
    ])
    def test_detect_arch(self, machine, expected_arch):
        with mock.patch("platform.machine", mock.MagicMock(return_value=machine)):
            result = detect_defaults_settings()
            result = dict(result)
            assert expected_arch == result['arch']

    @mock.patch("conan.internal.api.detect.detect_api.detect_clang_compiler",
                return_value=("clang", Version("9"), "clang"))
    def test_detect_clang_gcc_toolchain(self, _):
        output = RedirectedTestOutput()
        with redirect_output(output):
            with environment_update({"CC": "clang-9 --gcc-toolchain=/usr/lib/gcc/x86_64-linux-gnu/9"}):
                detect_defaults_settings()
                assert "CC and CXX: clang-9 --gcc-toolchain" in output


@pytest.mark.parametrize("version_return,expected_version", [
    ["cc.exe (Rev3, Built by MSYS2 project) 14.1.0", "14.1.0"],
    ["g++ (Conan-Build-gcc--binutils-2.42) 14.1.0", "14.1.0"],
    ["gcc.exe (x86_64-posix-seh-rev0, Built by MinGW-Builds project) 15.1.0", "15.1.0"],
    ["gcc.exe (x86_64-posix-seh-rev0, Built by MinGW-Builds project) 15.15.0", "15.15.0"],
    ["gcc (crosstool-NG 1.27.0) 13.3.0", "13.3.0"],
    ["gcc (crosstool-NG 1.27.0) 13.3", "13.3"],
    ["gcc (GCC) 4.8.1", "4.8.1"],
    ["clang version 18.1.0rc (https://github.com/llvm/llvm-project.git 461274b81d8641eab64d494accddc81d7db8a09e)", "18.1.0"],
    ["cc.exe (Rev3, Built by MSYS2 project) 14.0", "14.0"],
    ["clang version 18 (https://github.com/llvm/llvm-project.git 461274b81d8641eab64d494accddc81d7db8a09e)", "18"],
    ["cc.exe (Rev3, Built by MSYS2 project) 14", "14"],
    ["cc.exe 14.2", "14.2"],
    ["cc1 14.2", "14.2"],
])
@patch("conan.internal.api.detect.detect_api.detect_runner")
def test_detect_cc_versioning(detect_runner_mock, version_return, expected_version):
    detect_runner_mock.return_value = 0, version_return
    compiler, installed_version, compiler_exe = _cc_compiler()
    # Using private _value attribute to compare the str that reaches it, so no space issues arise
    assert installed_version._value == Version(expected_version)._value
    assert installed_version == Version(expected_version)


@pytest.mark.parametrize("function,version_return,expected_version", [
    [detect_suncc_compiler, "Sun C 5.13", ('sun-cc', Version("5.13"), 'cc')],
    [detect_intel_compiler, "Intel C++ Compiler 2025.0", ('intel-cc', Version("2025.0"), 'icx')],
])
def test_detect_compiler(function, version_return, expected_version):
    with mock.patch("conan.internal.api.detect.detect_api.detect_runner",
                    mock.MagicMock(return_value=(0, version_return))):
        ret = function()
        assert ret == expected_version



import platform
import textwrap
import pytest


new_value = "will_break_next"


@pytest.mark.skipif(platform.system() == "Windows", reason="Windows doesn't fail to link")
def test_auto_cppstd(matrix_c_interface_client):
    c = matrix_c_interface_client
    # IMPORTANT: This must be a C and CXX CMake project!!
    consumer = textwrap.dedent("""
        cmake_minimum_required(VERSION 3.15)
        project(myapp C CXX)

        find_package(matrix REQUIRED)

        add_executable(app app.c)
        target_link_libraries(app PRIVATE matrix::matrix)
        """)

    conanfile = textwrap.dedent("""\
        import os
        from conan import ConanFile
        from conan.tools.cmake import CMake, cmake_layout

        class Recipe(ConanFile):
            settings = "os", "compiler", "build_type", "arch"
            package_type = "application"
            generators = "CMakeToolchain", "CMakeDeps"
            requires = "matrix/0.1"

            def layout(self):
                cmake_layout(self)

            def build(self):
                cmake = CMake(self)
                cmake.configure()
                cmake.build()
                self.run(os.path.join(self.cpp.build.bindir, "app"), env="conanrun")
        """)
    app = textwrap.dedent("""
        #include "matrix.h"
        int main(){
            matrix();
            return 0;
        }
        """)
    c.save({"conanfile.py": conanfile,
            "CMakeLists.txt": consumer,
            "app.c": app}, clean_first=True)
    c.run(f"build . -c tools.cmake.cmakedeps:new={new_value}")
    assert "Hello Matrix!" in c.out



import textwrap
from pathlib import Path

from conan.tools.build.cpu import build_jobs
from jinja2 import Template

from conan.errors import ConanException
from conan.tools.files import save
from conan.tools.microsoft.msbuild import MSBuild
from conan.tools.premake.toolchain import PremakeToolchain
from conan.tools.premake.constants import CONAN_TO_PREMAKE_ARCH

# Source: https://learn.microsoft.com/en-us/cpp/overview/compiler-versions?view=msvc-170
PREMAKE_VS_VERSION = {
    '190': '2015',
    '191': '2017',
    '192': '2019',
    '193': '2022',
    '194': '2022',  # still 2022
    '195': '2026'
}

class Premake:
    """
    This class calls Premake commands when a package is being built. Notice that
    this one should be used together with the ``PremakeToolchain`` generator.

    This premake generator is only compatible with ``premake5``.
    """

    filename = "conanfile.premake5.lua"

    # Conan premake file which will preconfigure toolchain and then will call the user's premake file
    _premake_file_template = textwrap.dedent(
        """\
    #!lua
    include("{{luafile}}")
    include("{{premake_conan_toolchain}}")
    """
    )

    def __init__(self, conanfile):
        """
        :param conanfile: ``< ConanFile object >`` The current recipe object. Always use ``self``.
        """
        self._conanfile = conanfile
        #: Path to the root premake5 lua file (default is ``premake5.lua``)
        self.luafile = (Path(self._conanfile.source_folder) / "premake5.lua").as_posix()
        #: Key value pairs. Will translate to "--{key}={value}"
        self.arguments = {}  # https://premake.github.io/docs/Command-Line-Arguments/

        if "msvc" in self._conanfile.settings.compiler:
            msvc_version = PREMAKE_VS_VERSION.get(str(self._conanfile.settings.compiler.version))
            self.action = f'vs{msvc_version}'
        else:
            self.action = "gmake" # New generator (old gmakelegacy is deprecated)

        self._premake_conan_toolchain = Path(self._conanfile.generators_folder) / PremakeToolchain.filename

    @staticmethod
    def _expand_args(args):
        return ' '.join([f'--{key}={value}' for key, value in args.items()])

    def configure(self):
        """
        Runs ``premake5 <action> [FILE]`` which will generate respective build scripts depending on the ``action``.
        """
        if self._premake_conan_toolchain.exists():
            content = Template(self._premake_file_template).render(
                premake_conan_toolchain=self._premake_conan_toolchain.as_posix(), luafile=self.luafile
            )
            conan_luafile = Path(self._conanfile.build_folder) / self.filename
            save(self._conanfile, conan_luafile, content)
            arch = str(self._conanfile.settings.arch)
            if arch not in CONAN_TO_PREMAKE_ARCH:
                raise ConanException(f"Premake does not support {arch} architecture.")
            self.arguments["arch"] = CONAN_TO_PREMAKE_ARCH[arch]
        else:
            # Old behavior, for backward compatibility
            conan_luafile = self.luafile

        premake_options = dict()
        premake_options["file"] = f'"{conan_luafile}"'

        premake_command = (
            f"premake5 {self._expand_args(premake_options)} {self.action} "
            f"{self._expand_args(self.arguments)}{self._premake_verbosity}"
        )
        self._conanfile.run(premake_command)

    @property
    def _premake_verbosity(self):
        verbosity = self._conanfile.conf.get("tools.build:verbosity", choices=("quiet", "verbose"))
        return " --verbose" if verbosity == "verbose" else ""

    @property
    def _compilation_verbosity(self):
        verbosity = self._conanfile.conf.get("tools.compilation:verbosity", choices=("quiet", "verbose"))
        # --verbose does not print compilation commands but internal Makefile progress logic
        return " verbose=1" if verbosity == "verbose" else ""

    def build(self, workspace, targets=None, configuration=None, msbuild_platform=None):
        """
        Depending on the action, this method will run either ``msbuild`` or ``make`` with ``N_JOBS``.
        You can specify ``N_JOBS`` through the configuration line ``tools.build:jobs=N_JOBS``
        in your profile ``[conf]`` section.

        :param workspace: ``str`` Specifies the solution to be compiled (only used by ``MSBuild``).
        :param targets: ``List[str]`` Declare the projects to be built (None to build all projects).
        :param configuration: ``str`` Specify the configuration build type, default to build_type ("Release" or "Debug"),
            but this allow setting custom configuration type.
        :param msbuild_platform: ``str`` Specify the platform for the internal MSBuild generator (only used by ``MSBuild``).
        """
        if not self._premake_conan_toolchain.exists():
            raise ConanException("Premake.build() method requires PremakeToolchain to work properly")

        build_type = configuration or str(self._conanfile.settings.build_type)
        if self.action.startswith("vs"):
            msbuild = MSBuild(self._conanfile)
            if msbuild_platform:
                msbuild.platform = msbuild_platform
            msbuild.build_type = build_type
            msbuild.build(sln=f"{workspace}.sln", targets=targets)
        else:
            targets = "all" if targets is None else " ".join(targets)
            njobs = build_jobs(self._conanfile)
            self._conanfile.run(f"make config={build_type.lower()} {targets} -j{njobs}{self._compilation_verbosity}")



from conan.api.model import RecipeReference
from conans.server.rest.bottle_routes import BottleRoutes
from conans.server.rest.controller.v2 import get_package_ref
from conans.server.service.v2.service_v2 import ConanServiceV2
from conan.internal.util.dates import from_timestamp_to_iso8601


class RevisionsController(object):
    """
        Serve requests related with Conan
    """
    @staticmethod
    def attach_to(app):

        r = BottleRoutes()

        @app.route(r.recipe_revisions, method="GET")
        def get_recipe_revisions_references(name, version, username, channel, auth_user):
            """ Gets a JSON with the revisions for the specified recipe
            """
            conan_reference = RecipeReference(name, version, username, channel)
            conan_service = ConanServiceV2(app.authorizer, app.server_store)
            revs = conan_service.get_recipe_revisions_references(conan_reference, auth_user)
            return _format_revs_return(revs)

        @app.route(r.recipe_latest, method="GET")
        def get_latest_recipe_reference(name, version, username, channel, auth_user):
            """ Gets a JSON with the revisions for the specified recipe
            """
            conan_reference = RecipeReference(name, version, username, channel)
            conan_service = ConanServiceV2(app.authorizer, app.server_store)
            rev = conan_service.get_latest_revision(conan_reference, auth_user)
            return _format_rev_return(rev)

        @app.route(r.package_revisions, method="GET")
        def get_package_revisions_references(name, version, username, channel, package_id, auth_user,
                                             revision):
            """ Get a JSON with the revisions for a specified RREV """
            package_reference = get_package_ref(name, version, username, channel, package_id,
                                                revision, p_revision=None)
            conan_service = ConanServiceV2(app.authorizer, app.server_store)
            prefs = conan_service.get_package_revisions_references(package_reference, auth_user)
            return _format_prefs_return(prefs)

        @app.route(r.package_revision_latest, method="GET")
        def get_latest_package_reference(name, version, username, channel, package_id, auth_user,
                                         revision):
            """ Gets a JSON with the revisions for the specified recipe
            """
            package_reference = get_package_ref(name, version, username, channel, package_id,
                                                revision, p_revision=None)
            conan_service = ConanServiceV2(app.authorizer, app.server_store)
            pref = conan_service.get_latest_package_reference(package_reference, auth_user)
            return _format_pref_return(pref)


def _format_rev_return(rev):
    # FIXME: fix this when RecipeReference
    return {"revision": rev[0], "time": from_timestamp_to_iso8601(rev[1])}


def _format_revs_return(revs):
    return {"revisions": [_format_rev_return(rev) for rev in revs]}


def _format_pref_return(pref):
    return {"revision": pref.revision, "time": from_timestamp_to_iso8601(pref.timestamp)}


def _format_prefs_return(revs):
    return {"revisions": [_format_pref_return(rev) for rev in revs]}



import os
import platform
import textwrap

import pytest

from conan.test.utils.mocks import ConanFileMock
from conan.tools.env.environment import environment_wrap_command
from conan.test.assets.cmake import gen_cmakelists
from conan.test.assets.sources import gen_function_cpp


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
@pytest.mark.tool("cmake")
def test_transitive_headers_not_public(transitive_libraries):
    c = transitive_libraries

    conanfile = textwrap.dedent("""\
       from conan import ConanFile
       from conan.tools.cmake import CMake
       class Pkg(ConanFile):
           exports = "*"
           requires = "engine/1.0"
           settings = "os", "compiler", "arch", "build_type"
           generators = "CMakeToolchain", "CMakeDeps"

           def layout(self):
               self.folders.source = "src"

           def build(self):
               cmake = CMake(self)
               cmake.configure()
               cmake.build()
        """)
    cmake = gen_cmakelists(appsources=["main.cpp"], find_package=["engine"])
    main = gen_function_cpp(name="main", includes=["engine"], calls=["engine"])
    c.save({"src/main.cpp": main,
            "src/CMakeLists.txt": cmake,
            "conanfile.py": conanfile}, clean_first=True)

    c.run("build .")
    c.run_command(".\\Release\\myapp.exe")
    assert "matrix/1.0: Hello World Release!" in c.out

    # If we try to include transitivity matrix headers, it will fail!!
    main = gen_function_cpp(name="main", includes=["engine", "matrix"], calls=["engine"])
    c.save({"src/main.cpp": main})
    c.run("build .", assert_error=True)
    assert "Conan: Target declared 'matrix::matrix'" in c.out
    assert "Cannot open include file: 'matrix.h'" in c.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
@pytest.mark.tool("cmake")
def test_shared_requires_static(transitive_libraries):
    c = transitive_libraries

    conanfile = textwrap.dedent("""\
       from conan import ConanFile
       from conan.tools.cmake import CMake
       class Pkg(ConanFile):
           exports = "*"
           requires = "engine/1.0"
           default_options = {"engine/*:shared": True}
           settings = "os", "compiler", "arch", "build_type"
           generators = "CMakeToolchain", "CMakeDeps", "VirtualBuildEnv", "VirtualRunEnv"

           def layout(self):
               self.folders.source = "src"

           def build(self):
               cmake = CMake(self)
               cmake.configure()
               cmake.build()
        """)
    cmake = gen_cmakelists(appsources=["main.cpp"], find_package=["engine"])
    main = gen_function_cpp(name="main", includes=["engine"], calls=["engine"])
    c.save({"src/main.cpp": main,
            "src/CMakeLists.txt": cmake,
            "conanfile.py": conanfile}, clean_first=True)

    c.run("build .")
    command = environment_wrap_command(ConanFileMock(), "conanrun", c.current_folder, ".\\Release\\myapp.exe")
    c.run_command(command)
    assert "matrix/1.0: Hello World Release!" in c.out


@pytest.mark.skipif(platform.system() != "Windows", reason="Requires MSBuild")
@pytest.mark.tool("cmake")
def test_transitive_binary_skipped(transitive_libraries):
    c = transitive_libraries
    # IMPORTANT: matrix binary can be removed, no longer necessary
    c.run("remove matrix*:* -c")

    conanfile = textwrap.dedent("""\
       from conan import ConanFile
       from conan.tools.cmake import CMake
       class Pkg(ConanFile):
           exports = "*"
           requires = "engine/1.0"
           default_options = {"engine/*:shared": True}
           settings = "os", "compiler", "arch", "build_type"
           generators = "CMakeToolchain", "CMakeDeps"

           def layout(self):
               self.folders.source = "src"

           def build(self):
               cmake = CMake(self)
               cmake.configure()
               cmake.build()
        """)
    cmake = gen_cmakelists(appsources=["main.cpp"], find_package=["engine"])
    main = gen_function_cpp(name="main", includes=["engine"], calls=["engine"])
    c.save({"src/main.cpp": main,
            "src/CMakeLists.txt": cmake,
            "conanfile.py": conanfile}, clean_first=True)

    c.run("build . ")
    command = environment_wrap_command(ConanFileMock(), "conanrun", c.current_folder, ".\\Release\\myapp.exe")
    c.run_command(command)
    assert "matrix/1.0: Hello World Release!" in c.out

    # If we try to include transitivity matrix headers, it will fail!!
    main = gen_function_cpp(name="main", includes=["engine", "matrix"], calls=["engine"])
    c.save({"src/main.cpp": main})
    c.run("build .", assert_error=True)
    assert "Cannot open include file: 'matrix.h'" in c.out


@pytest.mark.tool("cmake")
def test_shared_requires_static_build_all(transitive_libraries):
    c = transitive_libraries

    conanfile = textwrap.dedent("""\
       from conan import ConanFile

       class Pkg(ConanFile):
           requires = "engine/1.0"
           settings = "os", "compiler", "arch", "build_type"
           generators = "CMakeDeps"
        """)

    c.save({"conanfile.py": conanfile}, clean_first=True)

    arch = c.get_default_host_profile().settings['arch']

    c.run("install . -o engine*:shared=True")
    assert not os.path.exists(os.path.join(c.current_folder, f"matrix-release-{arch}-data.cmake"))
    cmake = c.load(f"engine-release-{arch}-data.cmake")
    assert 'list(APPEND engine_FIND_DEPENDENCY_NAMES )' in cmake

    c.run("install . -o engine*:shared=True --build=engine*")
    assert not os.path.exists(os.path.join(c.current_folder, f"matrix-release-{arch}-data.cmake"))
    cmake = c.load(f"engine-release-{arch}-data.cmake")
    assert 'list(APPEND engine_FIND_DEPENDENCY_NAMES )' in cmake




from conan.internal.api.list.query_parse import _evaluate_postfix, _infix_to_postfix


class TestQueryParse:

    def test_get_postfix(self):
        r = _infix_to_postfix("")
        assert r == []

        r = _infix_to_postfix("a=2")
        assert r == ["a=2"]

        r = _infix_to_postfix("a=2 OR b=3")
        assert r == ["a=2", "b=3", "|"]

        r = _infix_to_postfix("a= OR b=")
        assert r == ["a=", "b=", "|"]  # Equivalent to ""

        r = _infix_to_postfix("(a=2 OR b=3) AND (j=34 AND j=45) OR (a=1)")
        assert r == ["a=2", "b=3", "|", "j=34", "j=45", "&", "a=1", "&", "|"]

        import pytest
        with pytest.raises(Exception, match="Invalid expression: 2"):
            r = _infix_to_postfix("a= 2 OR b=3")

    def test_evaluate_postfix(self):

        def evaluator(expr):
            return expr in ("a=2", "j=45")

        def evaluate(q):
            r = _infix_to_postfix(q)
            return _evaluate_postfix(r, evaluator)

        assert evaluate("a=2")
        assert not evaluate("a=4")
        assert evaluate("a=2 OR a=3")
        assert evaluate("a=4 OR j=45")
        assert not evaluate("a=4 AND j=45")
        assert evaluate("a=2 AND (f=23 OR j=45)")
        assert not evaluate("a=2 AND (f=23 OR j=435)")
        assert evaluate("a=2 AND j=45 OR h=23")
        assert evaluate("a=2 AND j=45 OR (h=23 AND a=2)")
        assert evaluate("((((a=2 AND ((((f=23 OR j=45))))))))")
        assert not evaluate("((((a=2 AND ((((f=23 OR j=42))))))))")



from unittest import mock
import pytest
import textwrap

from conan.internal.internal_tools import is_universal_arch
from conan.errors import ConanException
from conan.test.utils.mocks import ConanFileMock, MockSettings, MockOptions
from conan.test.utils.test_files import temp_folder
from conan.tools.apple import is_apple_os, to_apple_arch, fix_apple_shared_install_name, XCRun
from conan.tools.apple.apple import _get_dylib_install_name


def test_tools_apple_is_apple_os():
    conanfile = ConanFileMock()

    conanfile.settings = MockSettings({"os": "Macos"})
    assert is_apple_os(conanfile) == True

    conanfile.settings = MockSettings({"os": "watchOS"})
    assert is_apple_os(conanfile) == True

    conanfile.settings = MockSettings({"os": "Windows"})
    assert is_apple_os(conanfile) == False


def test_tools_apple_to_apple_arch():
    conanfile = ConanFileMock()

    conanfile.settings = MockSettings({"arch": "armv8"})
    assert to_apple_arch(conanfile) == "arm64"

    conanfile.settings = MockSettings({"arch": "x86_64"})
    assert to_apple_arch(conanfile) == "x86_64"


def test_fix_shared_install_name_no_libraries():
    conanfile = ConanFileMock()
    conanfile.options = MockOptions({"shared": True})
    conanfile.settings = MockSettings({"os": "Macos"})
    conanfile.folders.set_base_package(temp_folder())

    with pytest.raises(Exception) as e:
        fix_apple_shared_install_name(conanfile)
        assert "not found inside package folder" in str(e.value)


def test_xcrun_public_settings():
    # https://github.com/conan-io/conan/issues/12485
    conanfile = ConanFileMock()
    conanfile.settings = MockSettings({"os": "watchOS"})

    xcrun = XCRun(conanfile, use_settings_target=True)
    settings = xcrun.settings

    assert settings.os == "watchOS"


def test_get_dylib_install_name():
    # https://github.com/conan-io/conan/issues/13014
    single_arch = textwrap.dedent("""
    /path/to/libwebp.7.dylib:
    /absolute/path/lib/libwebp.7.dylib
    """)

    universal_binary = textwrap.dedent("""
    /.conan/data/package/lib/libwebp.7.dylib (architecture x86_64):
    /absolute/path/lib/libwebp.7.dylib
    /.conan/data/package/lib/libwebp.7.dylib (architecture arm64):
    /absolute/path/lib/libwebp.7.dylib
    """)

    for mock_output in (single_arch, universal_binary):
        with mock.patch("conan.tools.apple.apple.check_output_runner") as mock_output_runner:
            mock_output_runner.return_value = mock_output
            install_name = _get_dylib_install_name("otool", "/path/to/libwebp.7.dylib")
            assert "/absolute/path/lib/libwebp.7.dylib" == install_name


@pytest.mark.parametrize("settings_value,valid_definitions,result", [
    ("arm64|x86_64", ["arm64", "x86_64", "armv7", "x86"], True),
    ("x86_64|arm64", ["arm64", "x86_64", "armv7", "x86"], None),
    ("armv7|x86", ["arm64", "x86_64", "armv7", "x86"], True),
    ("x86|armv7", ["arm64", "x86_64", "armv7", "x86"], None),
    (None, ["arm64", "x86_64", "armv7", "x86"], False),
    ("arm64|armv7|x86_64", ["arm64", "x86_64", "armv7", "x86"], True),
    ("x86|arm64", ["arm64", "x86_64", "armv7", "x86"], None),
    ("arm64|ppc32", None, False),
    (None, None, False),
    ("armv7|x86", None, False),
    ("arm64", ["arm64", "x86_64"], False),
])
# None is for the exception case
def test_is_universal_arch(settings_value, valid_definitions, result):
    if result is None:
        with pytest.raises(ConanException):
            is_universal_arch(settings_value, valid_definitions)
    else:
        assert is_universal_arch(settings_value, valid_definitions) == result



import textwrap

import pytest

from conan.test.assets.sources import gen_function_c
from conan.test.utils.tools import TestClient

_conanfile_py = textwrap.dedent("""
    import os
    import shutil
    from conan import ConanFile
    from conan.tools.meson import Meson, MesonToolchain

    class App(ConanFile):
        settings = "os", "arch", "compiler", "build_type"
        options = {"shared": [True, False], "fPIC": [True, False], "use_french": ['true', 'false']}
        default_options = {"shared": False, "fPIC": True, "use_french": 'false'}
        exports_sources = "**"

        def config_options(self):
            if self.settings.os == "Windows":
                self.options.rm_safe("fPIC")

        def configure(self):
            if self.options.shared:
                self.options.rm_safe("fPIC")

        def layout(self):
            self.folders.build = "build"

        def generate(self):
            tc = MesonToolchain(self)
            tc.subproject_options["hello"] = [{'french': str(self.options.use_french)}]
            tc.generate()

        def build(self):
            meson = Meson(self)
            meson.configure()
            meson.build()

        def package(self):
            meson = Meson(self)
            meson.install()
            # https://mesonbuild.com/FAQ.html#why-does-building-my-project-with-msvc-output-static-libraries-called-libfooa
            if self.settings.compiler == 'msvc' and not self.options.shared:
                shutil.move(os.path.join(self.package_folder, "lib", "libhello.a"),
                            os.path.join(self.package_folder, "lib", "hello.lib"))
                shutil.move(os.path.join(self.package_folder, "lib", "libgreeter.a"),
                            os.path.join(self.package_folder, "lib", "greeter.lib"))

        def package_info(self):
            self.cpp_info.components["hello"].libs = ['hello']
            self.cpp_info.components["greeter"].libs = ['greeter']
            self.cpp_info.components["greeter"].requires = ['hello']
    """)

_meson_build = textwrap.dedent("""
    project('greeter', 'c')

    hello_proj = subproject('hello')
    hello_dep = hello_proj.get_variable('hello_dep')

    inc = include_directories('include')
    greeter = static_library('greeter',
        'greeter.c',
        include_directories : inc,
        dependencies : hello_dep,
        install : true)

    install_headers('greeter.h')
    """)

_meson_subproject_build = textwrap.dedent("""
    project('hello', 'c')

    hello_c_args = []
        if get_option('french')
        hello_c_args = ['-DFRENCH']
    endif

    inc = include_directories('include')
    hello = static_library('hello',
        'hello.c',
        include_directories : inc,
        c_args: hello_c_args,
        install : true)

    hello_dep = declare_dependency(include_directories : inc, link_with : hello)
    """)

_meson_subproject_options = textwrap.dedent("""
    option('french', type : 'boolean', value : false)
    """)

_hello_c = textwrap.dedent("""
    #include <stdio.h>

    void hello(void) {
    #ifdef FRENCH
        printf("Le sous-projet vous salut\\n");
    #else
        printf("Hello from subproject\\n");
    #endif
    }
    """)

_greeter_c = textwrap.dedent("""
    #include <hello.h>

    void greeter(void) {
        hello();
    }
    """)

_hello_h = textwrap.dedent("""
    #pragma once
    void hello(void);
    """)

_greeter_h = textwrap.dedent("""
    #pragma once
    void greeter(void);
    """)

_test_package_conanfile_py = textwrap.dedent("""
    import os
    from conan import ConanFile
    from conan.tools.cmake import CMake, cmake_layout
    from conan.tools.build import cross_building

    class TestConan(ConanFile):
        settings = "os", "compiler", "build_type", "arch"
        generators = "CMakeToolchain", "CMakeDeps"

        def requirements(self):
            self.requires(self.tested_reference_str)

        def layout(self):
            cmake_layout(self)

        def build(self):
            cmake = CMake(self)
            cmake.configure()
            cmake.build()

        def test(self):
            if not cross_building(self):
                cmd = os.path.join(self.cpp.build.bindirs[0], "test_package")
                self.run(cmd, env=["conanrunenv"])
    """)

_test_package_cmake_lists = textwrap.dedent("""
    cmake_minimum_required(VERSION 3.1)
    project(test_package C)

    add_executable(${PROJECT_NAME} src/test_package.c)
    find_package(greeter CONFIG REQUIRED)
    target_link_libraries(${PROJECT_NAME} greeter::greeter)
    """)


@pytest.mark.tool("ninja")
@pytest.mark.tool("cmake")
@pytest.mark.tool("meson")
def test_subproject():
    client = TestClient()
    test_package_c = gen_function_c(name="main", includes=["greeter"], calls=["greeter"])
    client.save({"conanfile.py": _conanfile_py,
                 "meson.build": _meson_build,
                 "greeter.c": _greeter_c,
                 "greeter.h": _greeter_h,
                 "include/greeter.h": _greeter_h,
                 "subprojects/hello/include/hello.h": _hello_h,
                 "subprojects/hello/hello.c": _hello_c,
                 "subprojects/hello/meson.build": _meson_subproject_build,
                 "subprojects/hello/meson_options.txt": _meson_subproject_options,
                 "test_package/conanfile.py": _test_package_conanfile_py,
                 "test_package/CMakeLists.txt": _test_package_cmake_lists,
                 "test_package/src/test_package.c": test_package_c})
    client.run("create . --name=greeter --version=0.1")
    assert "Hello from subproject" in client.out
    assert "Le sous-projet vous salut" not in client.out
    # Using subproject options
    client.run("create . --name=greeter --version=0.1 -o use_french='true'")
    assert "Hello from subproject" not in client.out
    assert "Le sous-projet vous salut" in client.out
